@@ -17,13 +17,13 @@ os.environ["SD_ENABLE_ASIO"] = "1"
 from . import analyzer, effect, exceptions, generator, io, typing  # noqa: E402
 from .afile import AFile  # noqa: E402
 from .asetup import ASetup  # noqa: E402
-from .interface import Interface, TestInterface  # noqa: E402
+from .interface import Interface  # noqa: E402
 
 if TYPE_CHECKING:
     import sounddevice as sd
 
 
-__all__ = ["Interface", "TestInterface", "ASetup", "AFile",
+__all__ = ["Interface", "ASetup", "AFile",
            "io", "generator", "effect", "analyzer", "typing", "exceptions"]
 
 # enable logging
