@@ -19,7 +19,7 @@ Higher memory usage of ACore functions can increase the thread switching time dr
 # Ensure that those are never imported!!!
 import queue
 from abc import ABC, abstractmethod
-from typing import TYPE_CHECKING, Optional, Tuple
+from typing import TYPE_CHECKING, Literal, Optional, Tuple
 
 import numpy as np
 import sounddevice as sd
@@ -42,7 +42,7 @@ class AGenerator(ABC):
         """
         self._out_buffer = out_buffer
         self._blocksize = blocksize
-        self._frame = start_frame
+        self._last_frame = start_frame
         self._start_frame = start_frame
 
         # set output channels and update _out_buf
@@ -74,13 +74,9 @@ class AGenerator(ABC):
             ch: The channel corresponding to the outdata at hand (zero indexed).
             frame: Current frame number to be processed, this always increases by +1 and is incremented by the callback.
         """
-        # if smaller than start frame, reset the generator
-        if frame < self._start_frame:
-            self._reset()
-            return
         # if the next frame is called, increment and set buffer reload flag
-        elif frame != self._frame:
-            self._frame = frame
+        if frame != self._last_frame:
+            self._last_frame = frame
             self._inc()
             self._reload = True
         # if out_buffer is enabled and reload flag is set fill _out_buf
@@ -97,6 +93,10 @@ class AGenerator(ABC):
         else:
             self._mod(outdata, ch)
 
+    def upreset(self) -> None:
+        self._last_frame = self._start_frame
+        self._reset()
+
     @abstractmethod
     def _mod(self, outdata: "ABlock", ch: int) -> None:
         """This function is envoked by `upstream()`.
@@ -120,10 +120,8 @@ class AGenerator(ABC):
 
     @abstractmethod
     def _reset(self) -> None:
-        """This function is envoked by `upstream()`.
-        It's called when the first frame is requested,
-        and should reset the generator into initial state."""
-        self._frame = self._start_frame
+        """This function should reset the generator into initial state."""
+        return None
 
 
 class AEffect(ABC):
@@ -143,7 +141,7 @@ class AEffect(ABC):
         self._in_buffer = in_buffer
         self._out_buffer = out_buffer
         self._blocksize = blocksize
-        self._frame = start_frame
+        self._last_frame = start_frame
         self._start_frame = start_frame
 
         # set in-/output channels and update buffers
@@ -191,13 +189,9 @@ class AEffect(ABC):
             ch: The channel corresponding to the outdata at hand (zero indexed).
             frame: Current frame number to be processed, this always increases by +1 and is incremented by the callback.
         """
-        # if smaller than start frame, reset the generator
-        if frame < self._start_frame:
-            self._reset()
-            return
         # if the next frame is called, increment and set buffer reload flag
-        elif frame != self._frame:
-            self._frame = frame
+        if frame != self._last_frame:
+            self._last_frame = frame
             self._inc()
             self._reload = True
         # if in_buffer is enabled and reload flag is set fill _in_buf
@@ -205,7 +199,7 @@ class AEffect(ABC):
             self._fill_in_buf(frame)
         # if out_buffer is enabled and reload flag is set fill _out_buf
         if self._out_buffer and self._reload:
-            self._fill_out_buf()
+            self._fill_out_buf(frame)
         self._reload = False
 
         # if out_buffer is enabled copy _out_buf
@@ -214,7 +208,16 @@ class AEffect(ABC):
         # otherwise process given array
         else:
             # _mod HAS TO HANDLE INPUT BUFFER + SETTING!!!
-            self._mod(outdata, ch)
+            self._mod(outdata, ch, frame)
+
+    def upreset(self) -> None:
+        # reset self
+        self._last_frame = self._start_frame
+        self._reset()
+        # call reset upstream
+        for in_a in self._in_as:
+            if in_a[0] is not None:
+                in_a[0].upreset()
 
     def _fill_in_buf(self, frame: int) -> None:
         self._in_buf.fill(0)
@@ -223,9 +226,9 @@ class AEffect(ABC):
                 # send _in_buf upstream
                 in_a[0].upstream(self._in_buf[:, in_ch], in_a[1], frame)
 
-    def _fill_out_buf(self) -> None:
+    def _fill_out_buf(self, frame: int) -> None:
         for out_ch in range(self._out_chs):
-            self._mod(self._out_buf[:, out_ch], out_ch)
+            self._mod(self._out_buf[:, out_ch], out_ch, frame)
 
     def _mod_in_buf(self, outdata: "ABlock", ch: int) -> None:
         """This function is envoked by `upstream()`.
@@ -243,7 +246,7 @@ class AEffect(ABC):
         """
         raise NotImplementedError("For enabled self._in_buffer, you must override this function!")
 
-    def _mod_upstream(self, outdata: "ABlock", ch: int) -> None:
+    def _mod_upstream(self, outdata: "ABlock", ch: int, frame: int) -> None:
         """This function is envoked by `upstream()`.
         It should write something in outdata for the given output channel ch.
         Make sure to copy your data or write directly into outdata and not just set outdata to a new reference.
@@ -252,6 +255,7 @@ class AEffect(ABC):
         Args:
             outdata: The 1D array to write to, with length blocksize.
             ch: The channel corresponding to the outdata at hand (zero indexed).
+            frame: Current frame number to be processed, this always increases by +1 and is incremented by the callback.
 
         Notes:
             The implementation of this function sends data upstream to process for each input channel.
@@ -261,18 +265,18 @@ class AEffect(ABC):
                 in_a = self._in_as[ch][0]
                 if in_a is not None:
                     # get outdata from upstream
-                    in_a.upstream(outdata, self._in_as[ch][1], self._frame)
+                    in_a.upstream(outdata, self._in_as[ch][1], frame)
                     # process outdata here
                     ...
             ```
         """
         raise NotImplementedError("For disabled self._in_buffer, you must override this function!")
 
-    def _mod(self, outdata: "ABlock", ch: int) -> None:
+    def _mod(self, outdata: "ABlock", ch: int, frame: int) -> None:
         if self._in_buffer:
             self._mod_in_buf(outdata, ch)
         else:
-            self._mod_upstream(outdata, ch)
+            self._mod_upstream(outdata, ch, frame)
 
     @abstractmethod
     def _inc(self) -> None:
@@ -284,10 +288,83 @@ class AEffect(ABC):
 
     @abstractmethod
     def _reset(self) -> None:
-        """This function is envoked by `upstream()`.
-        It's called when the first frame is requested,
-        and should reset the effect into initial state."""
-        self._frame = self._start_frame
+        """This function should reset the effect into initial state."""
+        return None
+
+
+class AEffectEnvelope(AEffect):
+    def __init__(self,
+                 gains: Tuple[float, ...],
+                 steps: Tuple[float, ...],
+                 scale: Literal["lin", "log"],
+                 in_buffer: bool,
+                 out_buffer: bool,
+                 blocksize: int,
+                 start_frame: int) -> None:
+        self.cue: queue.Queue[int] = queue.Queue(maxsize=1)
+
+        self._gains = gains
+        self._steps = steps
+        self._ramps = []
+        self._is_gain = self._gains[0]
+        for step in steps:
+            self._ramps.append(np.linspace(0, step, blocksize, dtype=np.float32))
+        self._scale = scale
+
+        self._idx = 0
+        self._gainramp = np.zeros(blocksize, dtype=np.float32)
+        super().__init__(in_buffer=in_buffer,
+                         out_buffer=out_buffer,
+                         blocksize=blocksize,
+                         start_frame=start_frame)
+
+    def update_ramp(self, start: int = 0) -> None:
+        idx = self._idx
+        if start > 0:
+            self._gainramp[start:] = self._is_gain + self._ramps[idx][:-start]
+            # print(self._gainramp[start-10:start+10])
+        else:
+            self._gainramp = self._is_gain + self._ramps[idx]
+        if self._steps[idx] > 0:
+            search = start + np.searchsorted(self._gainramp[start:], self._gains[idx], side="right")
+        elif self._steps[idx] < 0:
+            search = start + np.searchsorted(-self._gainramp[start:], -self._gains[idx], side="right")
+        else:
+            if not start and self._scale == "log":
+                self._log(self._gainramp)
+            return
+        # print("idx: ", self._idx, ", start: ", start, ", is_gain: ", self._is_gain, ", search: ", search)
+        if search < self._blocksize:
+            self._is_gain = self._gains[idx]
+            self._idx += 1
+            self.update_ramp(int(search))
+        self._is_gain = self._gainramp[-1]
+        if not start and self._scale == "log":
+            self._log(self._gainramp)
+
+    def _inc(self) -> None:
+        # test fo queues
+        if not self.cue.empty():
+            self._idx = self.cue.get()
+        self.update_ramp()
+
+    def _mod_in_buf(self, outdata: "ABlock", ch: int) -> None:
+        outdata[:] = self._in_buf[:, ch] * self._gainramp
+
+    def _mod_upstream(self, outdata: "ABlock", ch: int, frame: int) -> None:
+        in_a = self._in_as[ch][0]
+        if in_a is not None:
+            in_a.upstream(outdata, self._in_as[ch][1], frame)  # get outdata from upstream
+            outdata *= self._gainramp  # modify outdata
+
+    def _log(self, x: "ABlock") -> None:
+        x += np.log10(1 / 9)
+        np.power(10, x, out=x)
+        x -= 1 / 9
+
+    def _reset(self) -> None:
+        self._idx = 0
+        self._is_gain = self._gains[0]
 
 
 class AAnalyzer(ABC):
@@ -336,15 +413,18 @@ class AAnalyzer(ABC):
     def start_upstream(self, frame: int) -> None:
         # no additional frame processing necessary,
         # because this function is only called once per frame
-        if frame < self._start_frame:
-            self._reset()
-            return
         # if in_buffer is enabled fill _in_buf
         if self._in_buffer:
             self._fill_in_buf(frame)
             self._process_in_buf()
         else:
-            self._process_upstream()
+            self._process_upstream(frame)
+
+    def start_reset(self) -> None:
+        # call reset upstream
+        for in_a in self._in_as:
+            if in_a[0] is not None:
+                in_a[0].upreset()
 
     def _fill_in_buf(self, frame: int) -> None:
         self._in_buf.fill(0)
@@ -362,7 +442,7 @@ class AAnalyzer(ABC):
         """
         raise NotImplementedError("For enabled self._in_buffer, you must override this function.")
 
-    def _process_upstream(self) -> None:
+    def _process_upstream(self, frame: int) -> None:
         """This method is called once per audio frame to start the upstream chain.
 
         Notes:
@@ -373,16 +453,14 @@ class AAnalyzer(ABC):
             NUMPY_ARRAY_TO_WRITE_TO.fill(0)
             for in_ch, in_a in enumerate(self._in_as):
                 if in_a[0] is not None:
-                    in_a[0].upstream(NUMPY_ARRAY_TO_WRITE_TO[:, in_ch], in_a[1], self._frame)
+                    in_a[0].upstream(NUMPY_ARRAY_TO_WRITE_TO[:, in_ch], in_a[1], frame)
             ```
         """
         raise NotImplementedError("For disabled self._in_buffer, you must override this function.")
 
     @abstractmethod
     def _reset(self) -> None:
-        """This function is envoked by `upstream()`.
-        It's called when the first frame is requested,
-        and should reset the analyzer into initial state."""
+        """This function should reset the generator into initial state."""
         return None
 
 
@@ -436,7 +514,6 @@ class AAnalyzerBuf(AAnalyzer):
         self._in_buf_idx = 0
         while not self.buf_queue.empty():
             self.buf_queue.get(block=False)
-        return super()._reset()
 
 
 class AInterface:
@@ -451,10 +528,14 @@ class AInterface:
         self._out_buf: "ABuffer"
         self.out_ch_map = ()
         self.in_ch_map = ()
+        self._in_as: "InAs" = ()
 
         self._blocksize = blocksize
         self._frame = start_frame
-        self.ctime: Optional["CData"] = None
+        self._start_frame = start_frame
+        self.ctime: Optional["CTime"] = None
+
+        self._alzs: Tuple["AAnalyzer", ...] = ()
 
         self.end_frame: Optional[int] = None
 
@@ -510,20 +591,23 @@ class AInterface:
                  frames: int,
                  ctime: "CData",
                  status: sd.CallbackFlags) -> None:
-        if self.ctime is None:
-            self.ctime = ctime
-        # copy indata so it can be processed by upstream()
-        if self.out_ch_map:
-            self._out_buf[:] = indata[:, self.out_ch_map]
-        # call upstream method of the outputs connected to the inputs
-        if self.in_as != ():
-            outdata.fill(0)
-            for in_ch, in_a in enumerate(self._in_as):
-                if in_a[0] is not None:
-                    in_a[0].upstream(outdata[:, self.in_ch_map[in_ch]], in_a[1], self._frame)
-        # call AAnalyzers start_upstream method (because they wont get called otherwise)
-        for alz in self._alzs:
-            alz.start_upstream(self._frame)
+        # drop frames to allow callback to stabilize
+        if self._frame >= self._start_frame:
+            # ctime can be None for tests...
+            if self.ctime is None and ctime is not None:
+                self.ctime = CTime(ctime)
+            # copy indata so it can be processed by upstream()
+            if self.out_ch_map:
+                self._out_buf[:] = indata[:, self.out_ch_map]
+            # call upstream method of the outputs connected to the inputs
+            if self.in_as != ():
+                outdata.fill(0)
+                for in_ch, in_a in enumerate(self._in_as):
+                    if in_a[0] is not None:
+                        in_a[0].upstream(outdata[:, self.in_ch_map[in_ch]], in_a[1], self._frame)
+            # call AAnalyzers start_upstream method (because they wont get called otherwise)
+            for alz in self._alzs:
+                alz.start_upstream(self._frame)
         self._frame += 1  # Overflow?
         if self.end_frame is not None and self.end_frame <= self._frame:
             raise sd.CallbackStop
@@ -539,3 +623,30 @@ class AInterface:
             frame: Current frame number to be processed, this always increases by +1 and is incremented by the callback.
         """
         outdata[:] = self._out_buf[:, ch]
+
+    def reset(self, drop: int = 0) -> None:
+        """Reset all connected processors.
+
+        Args:
+            drop: Select how many frames to drop, when the audio stream restarts.
+        """
+        # reset self
+        self._frame = self._start_frame - drop
+        # call reset upstream
+        for in_a in self._in_as:
+            if in_a[0] is not None:
+                in_a[0].upreset()
+        # call AAnalyzers reset to start upstream reset
+        for alz in self._alzs:
+            alz.start_reset()
+
+    def upreset(self) -> None:
+        return
+
+
+class CTime():
+    def __init__(self, cdata: "CData"):
+        """Class used to parse and store sounddevice's CTime values."""
+        self.inputBufferAdcTime = cdata.inputBufferAdcTime
+        self.outputBufferDacTime = cdata.outputBufferDacTime
+        self.currentTime = cdata.currentTime
