@@ -63,7 +63,6 @@ class Recorder(Analyzer):
         def _reset(self) -> None:
             self._afile.flush()
             self._afile.seek(0)
-            return super()._reset()
 
 
 class Oscilloscope(Analyzer):
@@ -192,7 +191,6 @@ class Oscilloscope(Analyzer):
 
         def _reset(self) -> None:
             self._triggered = False
-            return super()._reset()
 
 
 class Vector(Analyzer):
@@ -277,10 +275,12 @@ class FFT(Analyzer):
     def update_acore(self) -> None:
         self._indata_fftw = pyfftw.empty_aligned((self._winsize, len(self._inputs)), dtype=np.float32)
         self._rfft = pyfftw.empty_aligned((int(self._winsize / 2 + 1), len(self._inputs)), dtype=np.complex64)
-        # TODO: This slows down the function, which is called for every dis-/conncet
+        # the FFTW_ESTIMATE flags skips the tests and relys on heuristics
+        # much faster connections, benchmarking did not show a slower fft.
         self._fftw = pyfftw.FFTW(self._indata_fftw,
                                  self._rfft,
-                                 axes=(0, ))
+                                 axes=(0, ),
+                                 flags=('FFTW_ESTIMATE', ))
         self._fft = np.zeros((int(self._winsize / 2 + 1), len(self._inputs)), dtype=np.complex64)
         self._tiledwin = np.tile(self._win[:, None], (1, len(self._inputs)))
         self._tmp = np.zeros((int(self._winsize / 2 + 1), len(self._inputs)), dtype=np.float32)
