@@ -1,12 +1,12 @@
 """In this submodule you can find all effects, so "audio processors" with one or multiple inputs and outputs."""
 import logging
 import queue
-from typing import TYPE_CHECKING, Literal
+from typing import TYPE_CHECKING, Literal, Tuple
 
 import numpy as np
 import pyfftw
 
-from .acore import AEffect
+from .acore import AEffect, AEffectEnvelope
 from .io import Input, Output
 from .processor import Effect
 
@@ -58,24 +58,24 @@ class Gain(Effect):
         def _mod_in_buf(self, outdata: "ABlock", ch: int) -> None:
             outdata[:] = self._in_buf[:, ch] * self._gain
 
-        def _mod_upstream(self, outdata: "ABlock", ch: int) -> None:
+        def _mod_upstream(self, outdata: "ABlock", ch: int, frame: int) -> None:
             in_a = self._in_as[ch][0]
             if in_a is not None:
-                in_a.upstream(outdata, self._in_as[ch][1], self._frame)  # get outdata from upstream
+                in_a.upstream(outdata, self._in_as[ch][1], frame)  # get outdata from upstream
                 outdata *= self._gain  # modify outdata
 
         def _inc(self) -> None:
-            return super()._inc()
+            return None
 
         def _reset(self) -> None:
-            return super()._reset()
+            return None
 
 
 class GainRamp(Effect):
     def __init__(self,
                  interface: "Interface",
                  gain: float,
-                 step: float,
+                 gradient: float,
                  scale: Literal["lin", "log"] = "lin",
                  in_buffer: bool = True,
                  out_buffer: bool = False) -> None:
@@ -89,12 +89,13 @@ class GainRamp(Effect):
         Args:
             interface: Reference to an Interface instance.
             gain: The initial gain.
-            step: The desired linear gain change per frame.
+            gradient: The desired linear gain change per second.
             scale: Scaling of the given gain.
             in_buffer: Flag that decides if inputs are buffered.
             out_buffer: Flag that decides if outputs are buffered.
         """
         self._gain = gain
+        step = gradient/interface.samplerate*interface.blocksize
         self._againramp = self._AGainRamp(gain, step, scale, in_buffer, out_buffer,
                                           interface.blocksize, interface.start_frame)
         super().__init__(aeffect=self._againramp,
@@ -124,6 +125,7 @@ class GainRamp(Effect):
                      out_buffer: bool,
                      blocksize: int,
                      start_frame: int) -> None:
+            # step: The desired linear gain change per frame.
             self.gain_queue: queue.Queue[float] = queue.Queue(maxsize=1)
 
             self._is_gain = gain
@@ -141,10 +143,10 @@ class GainRamp(Effect):
         def _mod_in_buf(self, outdata: "ABlock", ch: int) -> None:
             outdata[:] = self._in_buf[:, ch] * self._gainramp
 
-        def _mod_upstream(self, outdata: "ABlock", ch: int) -> None:
+        def _mod_upstream(self, outdata: "ABlock", ch: int, frame: int) -> None:
             in_a = self._in_as[ch][0]
             if in_a is not None:
-                in_a.upstream(outdata, self._in_as[ch][1], self._frame)  # get outdata from upstream
+                in_a.upstream(outdata, self._in_as[ch][1], frame)  # get outdata from upstream
                 outdata *= self._gainramp  # modify outdata
 
         def _inc(self) -> None:
@@ -178,9 +180,67 @@ class GainRamp(Effect):
         def _reset(self) -> None:
             self._is_gain = self._start_gain
             self._set_gain = self._start_gain
+            # clear queues
             while not self.gain_queue.empty():
                 self.gain_queue.get(block=False)
-            return super()._reset()
+
+
+class ADSR(Effect):
+    def __init__(self,
+                 interface: "Interface",
+                 attack: float,
+                 decay: float,
+                 sustain: float,
+                 release: float,
+                 scale: Literal["lin", "log"] = "lin",
+                 in_buffer: bool = True,
+                 out_buffer: bool = False) -> None:
+        assert attack > 0, "Attack must be > 0s."
+        # ((gain-setpoint, gradient), ...)
+        # init, attack
+        gains: Tuple[float, ...] = (0, 1)
+        steps: Tuple[float, ...] = (0, 1/attack/interface.samplerate*interface.blocksize)
+        # decay
+        if decay > 0:
+            gains += (sustain, )
+            steps += ((sustain - 1)/decay/interface.samplerate*interface.blocksize, )
+        else:
+            assert sustain == 1, "For zero decay, sustain must be 1."
+        # sustain
+        gains += (sustain, )
+        steps += (0, )
+        self._sustain_cue = len(gains)
+        # release
+        if release > 0:
+            gains += (0, 0)
+            steps += (-sustain/release/interface.samplerate*interface.blocksize, 0)
+        else:
+            self._sustain_cue -= 1
+            assert sustain == 0, "For zero release, sustain must be 0."
+        logger.info("gains = ", gains)
+        logger.info("steps = ", steps)
+        logger.info("sustain_cue = ", self._sustain_cue)
+        self._aadsr = AEffectEnvelope(gains, steps, scale, in_buffer, out_buffer,
+                                      interface.blocksize, interface.start_frame)
+
+        self._running = False
+        super().__init__(aeffect=self._aadsr,
+                         interface=interface,
+                         inputs=(Input(self), ),
+                         outputs=(Output(self), ),
+                         in_update=True,
+                         out_update=True)
+
+    def start(self) -> None:
+        self._aadsr.cue.put(1)
+        self._running = True
+
+    def running(self) -> bool:
+        return self._running
+
+    def release(self) -> None:
+        self._aadsr.cue.put(self._sustain_cue)
+        self._running = False
 
 
 class Sum(Effect):
@@ -218,10 +278,10 @@ class Sum(Effect):
             np.mean(self._in_buf, axis=1, out=outdata, dtype=np.float32)
 
         def _inc(self) -> None:
-            return super()._inc()
+            return None
 
         def _reset(self) -> None:
-            return super()._reset()
+            return None
 
 
 class Weight(Effect):
@@ -288,11 +348,13 @@ class Weight(Effect):
             self._rfft = pyfftw.empty_aligned(int(blocksize/2+1), dtype=np.complex64)
             self._fftw = pyfftw.FFTW(self._indata_fftw,
                                      self._rfft,
-                                     axes=(0, ))
+                                     axes=(0, ),
+                                     flags=('FFTW_ESTIMATE', ))
             self._ifftw = pyfftw.FFTW(self._rfft,
                                       self._indata_fftw,
                                       axes=(0, ),
-                                      direction="FFTW_BACKWARD")
+                                      direction="FFTW_BACKWARD",
+                                      flags=('FFTW_ESTIMATE', ))
 
             self._fs = np.fft.rfftfreq(blocksize, 1/samplerate).astype(np.float32)
 
@@ -352,11 +414,11 @@ class Weight(Effect):
             else:
                 outdata[:] = self._in_buf[:, ch]
 
-        def _mod_upstream(self, outdata: "ABlock", ch: int) -> None:
+        def _mod_upstream(self, outdata: "ABlock", ch: int, frame: int) -> None:
             in_a = self._in_as[ch][0]
             if in_a is not None:
                 # get outdata from upstream
-                in_a.upstream(outdata, self._in_as[ch][1], self._frame)
+                in_a.upstream(outdata, self._in_as[ch][1], frame)
                 if self._weight:
                     # process outdata here
                     self._indata_fftw[:] = outdata
@@ -367,7 +429,7 @@ class Weight(Effect):
                     outdata[:] = self._indata_fftw
 
         def _inc(self) -> None:
-            return super()._inc()
+            return None
 
         def _reset(self) -> None:
-            return super()._reset()
+            return None
