@@ -1,5 +1,5 @@
 import logging
-from typing import TYPE_CHECKING, Any, Dict, Literal, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Dict, Optional, Tuple, Unpack
 
 import sounddevice as sd
 
@@ -13,7 +13,7 @@ if TYPE_CHECKING:
     from .asetup import ASetup
     from .exceptions import DeviceError
     from .processor import Analyzer
-    from .typing import Callback, Device, InAs
+    from .typing import Callback, Device, InAs, InterfaceIOTD
 
 logger = logging.getLogger(__name__)
 
@@ -182,6 +182,10 @@ class Interface(Processor):
     def outputs(self) -> Tuple["IInput", ...]:
         return self._iinputs
 
+    @property
+    def inputs(self) -> Tuple["IOutput", ...]:
+        return self._ioutputs
+
     @property
     def iinputs(self) -> Tuple["IInput", ...]:
         """Returns a list of all IInputs."""
@@ -195,24 +199,23 @@ class Interface(Processor):
     def iinput(self,
                idx: int = 0,
                ch: Optional[int] = None,
-               name: Optional[str] = None,
-               ref: Optional[Literal[True]] = None) -> "IInput":
+               **kwargs: Unpack["InterfaceIOTD"]) -> "IInput":
         """If the given index points to the next element of iinputs list,
         or if the index exists, but the given channel does not:
-        Create new IInput for given channel and return it.
-        Otherwise, search for the IInput by given arguments in the order ch, name, ref, idx and return it.
+        Create new IInput for given channel, with given kwargs, and return it.
+        Otherwise, search for the IInput matching one of the given kwargs.
+        Search is conducted in the given order.
 
         Args:
             idx: Index in zero indexed list of IInputs.
             ch: Interface analog input channel, stored in the IInput.
-            name: Given name of the channel, stored in the IInput.
-            ref: The reference IInput.
+            **kwargs (InterfaceIOTD): Optional attributes, additional paramters are possible.
 
         Raises:
             ValueError: Given idx does not point to the next free slot:
                 Cant create more than one channel at once.
             ValueError: New IInput must specify a physical channel. Use the ch argument.
-            ValueError: No IInput registered for given argument.
+            ValueError: No IInput with given kwargs registered.
 
         Returns:
             Reference to IInput object.
@@ -227,7 +230,7 @@ class Interface(Processor):
                 raise ValueError("New IInput must specify a physical channel. Use the ch argument.")
             logger.info("Index points to next elemnt in iinputs list: "
                         "Create new IInput.")
-            iinput = IInput(self, ch, name=name, reference=ref)
+            iinput = IInput(self, ch, **kwargs)
             self._iinputs += (iinput, )
             return iinput
         # idx exists, continue and check channel
@@ -236,58 +239,54 @@ class Interface(Processor):
                 return next((outpu for outpu in self._iinputs if outpu.channel == ch))
             except StopIteration:
                 logger.info("Channel not found, create new IInput.")
-                iinput = IInput(self, ch, name=name, reference=ref)
+                iinput = IInput(self, ch, **kwargs)
                 self._iinputs += (iinput, )
                 return iinput
-        # search for name
-        if name is not None:
-            try:
-                return next((outpu for outpu in self._iinputs if outpu.name == name))
-            except StopIteration as exc:
-                raise ValueError(f"No IInput with name {name} registered.") from exc
-        # search for reference
-        if ref is not None:
-            try:
-                return next((outpu for outpu in self._iinputs if outpu.reference))
-            except StopIteration as exc:
-                raise ValueError("No reference IInput registered.") from exc
+        # search for kwargs
+        if kwargs:
+            for key, name in kwargs.items():
+                try:
+                    return next((outpu for outpu in self._iinputs if getattr(outpu, key, None) == name))
+                except StopIteration:
+                    pass
+            raise ValueError("No IInput with given kwargs registered.")
         return self._iinputs[idx]
 
     def ioutput(self,
                 idx: int = 0,
                 ch: Optional[int] = None,
-                name: Optional[str] = None,
-                ref: Optional[Literal[True]] = None) -> "IOutput":
+                **kwargs: Unpack["InterfaceIOTD"]) -> "IOutput":
         """If the given index points to the next element of ioutputs list,
         or if the index exists, but the given channel does not:
-        Create new IOutput for given channel and return it.
-        Otherwise, search for the IOutput by given arguments in the order ch, name, ref, idx and return it.
+        Create new IOutput for given channel, with given kwargs, and return it.
+        Otherwise, search for the IInput matching one of the given kwargs.
+        Search is conducted in the given order.
 
         Args:
             idx: Index in zero indexed list of IOutputs.
             ch: Interface analog output channel, stored in the IOutput.
-            name: Given name of the channel, stored in the IOutput.
-            ref: The reference IInput.
+            **kwargs (InterfaceIOTD): Optional attributes, additional paramters are possible.
 
         Raises:
             ValueError: Given idx does not point to the next free slot:
                 Cant create more than one channel at once.
-            ValueError: New IInput must specify a physical channel. Use the ch argument.
-            ValueError: No IInput registered for given argument.
+            ValueError: New IOutput must specify a physical channel. Use the ch argument.
+            ValueError: No IInput with given kwargs registered.
 
         Returns:
             Reference to IOutput object.
         """
+        # handle idx
         if idx > len(self._ioutputs):
             raise ValueError("Given idx does not point to the next free slot: "
                              "Cant create more than one channel at once.")
-        # new iinput, parse other parameters
+        # new ioutput, parse other parameters
         if idx == len(self._ioutputs):
             if ch is None:
                 raise ValueError("New IOutput must specify a physical channel. Use the ch argument.")
-            logger.info("Index points to next elemnt in iinputs list: "
+            logger.info("Index points to next elemnt in ioutputs list: "
                         "Create new IOutput.")
-            ioutput = IOutput(self, ch, name=name, reference=ref)
+            ioutput = IOutput(self, ch, **kwargs)
             self._ioutputs += (ioutput, )
             return ioutput
         # idx exists, continue and check channel
@@ -296,21 +295,17 @@ class Interface(Processor):
                 return next((inpu for inpu in self._ioutputs if inpu.channel == ch))
             except StopIteration:
                 logger.info("Channel not found, create new IOutput.")
-                ioutput = IOutput(self, ch, name=name, reference=ref)
+                ioutput = IOutput(self, ch, **kwargs)
                 self._ioutputs += (ioutput, )
                 return ioutput
-        # search for name
-        if name is not None:
-            try:
-                return next((inpu for inpu in self._ioutputs if inpu.name == name))
-            except StopIteration as exc:
-                raise ValueError(f"No IOutput with name {name} registered.") from exc
-        # search for reference
-        if ref is not None:
-            try:
-                return next((inpu for inpu in self._ioutputs if inpu.reference))
-            except StopIteration as exc:
-                raise ValueError("No reference IOutput registered.") from exc
+        # search for kwargs
+        if kwargs:
+            for key, name in kwargs.items():
+                try:
+                    return next((inpu for inpu in self._ioutputs if getattr(inpu, key, None) == name))
+                except StopIteration:
+                    pass
+            raise ValueError("No IInput with given kwargs registered.")
         return self._ioutputs[idx]
 
     def get_latency(self) -> int:
@@ -318,7 +313,7 @@ class Interface(Processor):
 
         !!! warning
             Dont rely on this method, as it only calculates the ADC/DAC's internal latency.
-            Use [calibrate_latency.py](../examples.md/#calibrate_latency.py)
+            Use [calibrate_latency.py](../examples/calibration.md/#calibrate_latency.py)
             to compare this result with the real loopback calibration.
 
         Raises:
@@ -432,20 +427,21 @@ class Interface(Processor):
                 return
 
         if self._iinputs and not self._ioutputs:
-            stream.channels = max([inpu.channel for inpu in self._iinputs])
+            stream.channels = (max([inpu.channel for inpu in self._iinputs]), 1)
         if self._ioutputs and not self._iinputs:
-            stream.channels = max([output.channel for output in self._ioutputs])
+            stream.channels = (1, max([output.channel for output in self._ioutputs]))
         if self._iinputs and self._ioutputs:
             stream.channels = (max([inpu.channel for inpu in self._iinputs]),
                                max([output.channel for output in self._ioutputs]))
 
-    def start(self, end_frame: Optional[int] = None, reset: bool = True) -> sd.Stream:
+    def start(self, end_frame: Optional[int] = None, reset: bool = True, drop: int = 3) -> sd.Stream:
         """Start the audio stream.
 
         Args:
             end_frame: If set, the stream is stopped at the given end_frame.
             reset: If Truet, all audio processors are reset to initial frame.
                 Otherwise, the Stream continues after the last stop.
+            drop: Select how many frames to drop, when the audio stream restarts.
 
         Returns:
             Reference to the started sounddevice stream.
@@ -464,10 +460,8 @@ class Interface(Processor):
         """
         self._init_sounddevice()
         self._ainterface.end_frame = end_frame
-        # drop first three frames on reset (used for reset)
-        # also recommended for first start
         if reset:
-            self._ainterface.frame = self.start_frame - 3
+            self._ainterface.reset(drop=drop)
         stream = sd.Stream(callback=self._ainterface.callback)
         stream.start()
         return stream
