@@ -1,18 +1,15 @@
-from typing import TYPE_CHECKING, Annotated
+from typing import TYPE_CHECKING
 
 import numpy as np
-import numpy.typing as npt
 import scipy as sp
 
 if TYPE_CHECKING:
-    Data = Annotated[npt.NDArray[np.float32], "Blocksize x Channels"]
-    RefData = Annotated[npt.NDArray[np.float32], "Blocksize"]
-    Peaks = Annotated[npt.NDArray[np.float32], "Channels"]
+    from asmu.typing import ABlock, ABuffer, Avg
 
 
-def get_corrs_sampleshifts(indata: "Data",
-                           refdata: "RefData",
-                           burstLength: int) -> tuple["Data", "Peaks"]:
+def get_corrs_sampleshifts(indata: "ABuffer",
+                           refdata: "ABlock",
+                           burstLength: int) -> tuple["ABuffer", "Avg"]:
     """This function calculates the crosscorrelation between all channels of the indata relative to refdata.
     Additionally the maximum peak is exported.
 
