@@ -38,7 +38,7 @@ if __name__ == "__main__":
     asetup.interface = interface
 
     if input("Calibrate? (y|n)") == "y":
-        calibrate_iinput_cPa(interface, in_ch)
+        calibrate_iinput_cPa(interface, interface.iinput(ch=9))
         asetup.save()
     else:
         asetup.load()
